﻿using System.ComponentModel;
using System.Reflection;

namespace IC_DB
{
   /// <summary>
   /// Auflistung der verschiedenen Pin-Typen.
   /// </summary>
   /// <remarks>Das Attribut <see cref="DescriptionAttribute"/> dient zur Anzeige eines Kurztextes im Pin-Editor.<para/>
   /// Das Attribut <see cref="LogicLevelAttribute"/>legt bei den funktionalen Pins einzustellenden logischen Wert fest.</remarks>
   public enum PinType // Muss mit enum IcPinTypes_t in TestIc.h identisch sein!
   {
      /// <summary>Nicht angeschlossen.</summary>
      [Description("NC")] [LogicLevel(LogicLevel.Low)] NC,
      /// <summary>negative Versorgungsspannung.</summary>
      [Description("GND")][LogicLevel(LogicLevel.Low)] GND,
      /// <summary>positive Versorgungsspannung (+5V).</summary>
      [Description("VCC")][LogicLevel(LogicLevel.High)] VCC,
      /// <summary>Der Pin ist ein INPUT-Pin.</summary>
      [Description("Inp")] Input,
      /// <summary>Der Pin ist ein OUTPUT-Pin.</summary>
      [Description("Out")] Output,
      //     IO, Später
      /// <summary>Der Pin ist ein OUTPUT-Pin mit OpenCollector-Ausgang.</summary>
      [Description("OC")] OpenCollector,
      /// <summary>Data-Pin der I²C-Schnittstelle.</summary>
      [Description("SDA")][LogicLevel(LogicLevel.Low)] SDA,
      /// <summary>Clock-Pin der I²C-Schnittstelle.</summary>
      [Description("SCL")][LogicLevel(LogicLevel.Low)] SCL

      // Bei Änderungen muss u.a. SendTestPacket überarbeitet werden.
   }

   /// <summary>
   /// Klasse mit Ergänzungsmethoden zur <see cref="PinType"/>-Enumeration.
   /// </summary>
   public static class PinTypeMethods
   {
      /// <summary>
      /// Liefert die Beschreibung des Pins.
      /// </summary>
      /// <param name="value">Das <see cref="PinType"/>-Objekt, dessen Beschreibung (<see cref="DescriptionAttribute"/>) geliefert werden soll.</param>
      /// <returns>Die Beschreibung des Pins.</returns>
      public static string? GetDescription(this PinType value)
      {
         FieldInfo? fieldInfo = value.GetType().GetField(value.ToString());
         if (fieldInfo == null)
            return null;
         DescriptionAttribute? attribute = (DescriptionAttribute?)fieldInfo.GetCustomAttribute(typeof(DescriptionAttribute));
         if (attribute == null)
            return null;
         return attribute.Description;
      }

      /// <summary>
      /// Liefert den logischen Wert für funktionale Pins (siehe <see cref="IsFunctionalPin(PinType)"/>
      /// </summary>
      /// <param name="value"></param>
      /// <returns>Der zu setzende logische Wert.</returns>
      /// <exception cref="ArgumentException">Nicht funktionale Pins besitzen keinen Stanfdard-Wert. 
      /// Er wird beim Testschritt definiert.</exception>
      public static LogicLevel GetLogicLevel(this PinType value)
      {
         return LogicLevelAttribute.GetLogicLevel(value);
      }

      /// <summary>
      /// Gibt an, ob der Pin eine Sonderfunktion wie GND oder SDA besitzt.
      /// </summary>
      /// <param name="value">Das <see cref="PinType"/>-Objekt, das geprüft werden soll.</param>
      /// <returns><b>true</b>, wenn der Pin ein I2C-Schnittstellen-Pin ist, ansonsten <b>false</b>.</returns>
      public static bool IsFunctionalPin(this PinType value)
      {
         switch (value)
         {
            case PinType.NC:
               return true;
            case PinType.GND:
               return true;
            case PinType.VCC:
               return true;
            case PinType.Input:
               return false;
            case PinType.Output:
               return false;
            case PinType.OpenCollector:
               return false;
            case PinType.SDA:
               return true;
            case PinType.SCL:
               return true;
            default:
               throw new NotImplementedException("IsFunctionalPin");
         }
      }
   }
}