﻿using System.ComponentModel;
using System.Reflection;
using System.Runtime.InteropServices;


namespace IC_Tester_Controls
{
   /// <summary>
   /// Legt die Sortierung der DropDown-Liste fest.
   /// </summary>
   public enum SelectorSorting
   {
      /// <summary>Keine Sortierung.</summary>
      None,
      /// <summary>Alphabetische Sortierung.</summary>
      Alphabetical,
      /// <summary>Nummerische Sortierung.</summary>
      Numerical
   }

   /// <summary>
   /// Steuerelement zur Auswahl eines Enumerationswerts.
   /// </summary>
   /// <typeparam name="T">Der Enumrationstyp aus dem ein Wert ausgewählt werden soll.</typeparam>
   /// <remarks>Die Anzeigetexte werden aus dem <see cref="DescriptionAttribute"/> der Enumerationswerte gewonnen.</remarks>
   public partial class EnumSelector<T> : UserControl where T : Enum
   {
      private HorizontalAlignment mTextAlignment = HorizontalAlignment.Left;
      private TextFormatFlags dropDownFlags = TextFormatFlags.Left;
      private int textStyle = 0;
      private SelectorSorting mSortBy = SelectorSorting.None;

      /// <summary>
      /// Initialisiert eine neue Instanz der <see cref="EnumSelector{T}"/>-Klasse.
      /// </summary>
      public EnumSelector()
      {
         if (!typeof(T).IsEnum)
         {
            throw new ArgumentException("T must be an enumerated type");
         }
         InitializeComponent();
         FillComboBoxItems();
      }

      private void FillComboBoxItems()
      {
         comboBox1.Items.Clear();
         var items = new List<ListItem<T>>();

         foreach (T enumValue in Enum.GetValues(typeof(T)))
            items.Add(new ListItem<T>(enumValue));

         switch (mSortBy)
         {
            case var @case when @case == SelectorSorting.None:
               {
                  break;
               }
            case var case1 when case1 == SelectorSorting.Alphabetical:
               {
                  items.Sort((x, y) => x.Text.CompareTo(y.Text));
                  break;
               }
            case var case2 when case2 == SelectorSorting.Numerical:
               {
                  items.Sort((x, y) => x.EnumField.CompareTo(y.EnumField));
                  break;
               }

            default:
               {
                  break;
               }
         }

         comboBox1.Items.AddRange(items.ToArray());
      }

      #region [ Events ]
      /// <summary>Tritt ein, wenn die Selektion geäöndert wurde.</summary>
      public event SelectionChangedEventHandler? SelectionChanged;

      /// <summary>
      /// Tritt ein, wenn die Selektion geändert wurde.
      /// </summary>
      /// <param name="sender">Die Quelle des Ereignisses.</param>
      /// <param name="enumValue">Der ausgewählte Enumerationswert.</param>
      public delegate void SelectionChangedEventHandler(object sender, T enumValue);
      #endregion

      #region [ Properties ]
      /// <summary>
      /// Ruft die Textausrichtung ab oder legt sie fest.
      /// </summary>
      [Category("Appearance")]
      [DefaultValue(typeof(HorizontalAlignment), "Left")]
      [Description("Ruft die Textausrichtung ab oder legt sie fest.")]
      public HorizontalAlignment TextAlignment
      {
         get
         {
            return mTextAlignment;
         }
         set
         {
            mTextAlignment = value;

            switch (value)
            {
               case var @case when @case == HorizontalAlignment.Left:
                  {
                     textStyle = 0;
                     dropDownFlags = TextFormatFlags.Left;
                     break;
                  }
               case var case1 when case1 == HorizontalAlignment.Right:
                  {
                     textStyle = 2;
                     dropDownFlags = TextFormatFlags.Right;
                     break;
                  }
               case var case2 when case2 == HorizontalAlignment.Center:
                  {
                     textStyle = 1;
                     dropDownFlags = TextFormatFlags.HorizontalCenter;
                     break;
                  }

               default:
                  {
                     break;
                  }
            }

            ComboBoxExtension.SetTextAlignment(comboBox1.Handle, textStyle);
            comboBox1.Refresh();
         }
      }

      /// <summary>
      /// Ruft die Sortierung ab oder legt sie fest.
      /// </summary>
      [Category("Appearance")]
      [DefaultValue(typeof(SelectorSorting), "None")]
      [Description("Ruft die Sortierung ab oder legt sie fest.")]
      public SelectorSorting SortBy
      {
         get
         {
            return mSortBy;
         }
         set
         {
            mSortBy = value;
            FillComboBoxItems();
         }
      }

      /// <summary>
      /// Ruft den Index des derzeit ausgewählten Elements ab oder legt diesen fest.
      /// </summary>
      /// <returns>Ein nullbasierter Index des derzeit ausgewählten Elements. Wenn kein Element ausgewählt ist, wird ein Wert von -1 zurückgegeben..</returns>
      [Category("Appearance")]
      [DefaultValue(-1)]
      [Description("Ruft den Index des derzeit ausgewählten Elements ab oder legt diesen fest.")]
      [Browsable(false)]
      public int SelectedIndex
      {
         get
         {
            return comboBox1.SelectedIndex;
         }
         set
         {
            comboBox1.SelectedIndex = value;
         }
      }

      /// <summary>
      /// Ruft das derzeit ausgewählte Element ab oder legt es fest.
      /// </summary>
      /// <returns>Das derzeit ausgewählte Element.</returns>
      [Browsable(false)]
      [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
      public T SelectItem
      {
         get
         {
            return ((ListItem<T>)comboBox1.SelectedItem).EnumField;
         }
         set
         {
            for (int i = 0, loopTo = comboBox1.Items.Count - 1; i <= loopTo; i++)
            {
               ListItem<T> li = (ListItem<T>)comboBox1.Items[i];
               if (li.EnumField.Equals(value))
               {
                  comboBox1.SelectedIndex = i;
                  return;
               }
            }
            throw new InvalidEnumArgumentException();
         }
      }
      #endregion

      #region "[ Event-Handler ]"
      private void EnumSelector_Load(object sender, EventArgs e)
      {
         if (DesignMode)
         {
            return;
         }

         // Das UserControl kan mit einem ToolTip versehen sein. Dieses
         // wird aber nicht angezeigt, weil die ComboBox die gesamte Fläche füllt.
         // Der ToolTip-Text muss also an die ComboBox weiter gegegeben werden.
         // Der Vorgang muss über Reflection erledigt werden, da das ToolTip-Objekt
         // als Private deklariert dein kann.

         // Prüfen, ob im Form ein ToolTip-Objekt angelegt ist

         // Die Container-Hierrarchie bis zum Form aufsteigen.
         var p = this.Parent;
         FieldInfo? toolTipFieldInfo = null;

         while (!p.GetType().IsSubclassOf(typeof(Form)))
            p = p.Parent;

         // Gibt es ein (ggf. privates) Feld der Klasse ToolTip?
         var bf = BindingFlags.Instance | BindingFlags.NonPublic | BindingFlags.Public;

         foreach (FieldInfo f in p.GetType().GetFields(bf))
         {

            if (f.FieldType == typeof(ToolTip))
            {
               toolTipFieldInfo = f;
               break;
            }
         }

         // ToolTip gefunden, Text weiter reichen.
         if (toolTipFieldInfo is not null)
         {
            var valueOfField = toolTipFieldInfo.GetValue(p);
            var customControlType = toolTipFieldInfo.FieldType;
            var methodInfo = customControlType.GetMethod("GetToolTip");
            if (methodInfo != null)
            {
               var m = methodInfo.Invoke(valueOfField, new object[] { this });
               if (m != null)
               {
                  string toolTipText = (String)m;
                  toolTip1.SetToolTip(comboBox1, toolTipText);
               }
            }
         }
      }

      private void comboBox1_DrawItem(object sender, DrawItemEventArgs e)
      {
         e.DrawBackground();
         string? txt = "";
         if (e.Index >= 0)
            txt = comboBox1.Items[e.Index].ToString();
         TextRenderer.DrawText(e.Graphics, txt, e.Font, e.Bounds, e.ForeColor, dropDownFlags);
         e.DrawFocusRectangle();
      }
      /// <summary>Leitet das SelectedIndexChanged-Ereignis weiter.</summary>
      private void comboBox1_SelectedIndexChanged(object sender, EventArgs e)
      {
         SelectionChanged?.Invoke(this, ((ListItem<T>)comboBox1.SelectedItem).EnumField);
      }
   }
   #endregion

   #region  Class ListItem 
   /// <summary>
   /// Klasse zur Darstellung von Items listenartigen Steuerelementen mit Text und Wert.
   /// </summary>
   internal partial class ListItem<Q> where Q : Enum
   {
      /// <summary>Anzuzeigender Text.</summary>
      public readonly string Text;
      /// <summary>Dem Item zu Grunde liegender Wert.</summary>
      public readonly Q EnumField;

      /// <summary>
      /// Initialisiert eine neue Instanz der <see cref="ListItem{Q}"/>-Klasse.
      /// </summary>
      /// <param name="EnumField">Dem Item zu Grunde liegender Wert.</param>
      public ListItem(Q EnumField)
      {
         Text = GetEnumDescription(EnumField);
         this.EnumField = EnumField;
      }

      /// <summary>
      /// Liefert den im listenartigen Steuerelementen anzuzeigenden Text.
      /// </summary>
      /// <returns>Der im listenartigen Steuerelementen anzuzeigende Text.</returns>
      public override string ToString()
      {
         return Text;
      }

      /// <summary>
      /// Liest das <see cref="DescriptionAttribute"/> des Enumerationswertes aus.
      /// </summary>
      /// <param name="value">Der Enumerationswerte, dessen <see cref="DescriptionAttribute"/> ermittelt werden soll.</param>
      /// <returns>Der beim Enumerationswert hintelegte Beschreibungstext oder der Name des Enumerationswerts.</returns>
      public static string GetEnumDescription(Q value)
      {

         FieldInfo? fi = value.GetType().GetField(value.ToString());

         if (fi != null)
         {
            DescriptionAttribute[] attributes = (DescriptionAttribute[])fi.GetCustomAttributes(typeof(DescriptionAttribute), false);
            if (attributes != null && attributes.Length > 0)
            {
               return attributes[0].Description;
            }
         }
         return value.ToString();
      }
   }

   #endregion


   #region [ Hilfsklasse ComboBoxExtension ]
   /// <summary>
   /// Erweiterungsfunktionen für eine ComboBox.
   /// </summary>
   /// <remarks>DLLImport ist in einer generische Klasse nicht zulässig. 
   /// 'setTextAlignment' muss deshalb in einer separaten Klasse implementiert werden.</remarks>
   internal static class ComboBoxExtension
   {
      [StructLayout(LayoutKind.Sequential)]
      private partial struct RECT
      {
         public int Left;
         public int Top;
         public int Right;
         public int Bottom;
      }

      [StructLayout(LayoutKind.Sequential)]
      private partial struct COMBOBOXINFO
      {
         public int cbSize;
         public RECT rcItem;
         public RECT rcButton;
         public int stateButton;
         public IntPtr hwndCombo;
         public IntPtr hwndEdit;
         public IntPtr hwndList;
      }

      [DllImport("user32.dll")]
      private static extern bool GetComboBoxInfo(IntPtr hWnd, ref COMBOBOXINFO pcbi);
      [DllImport("user32.dll")]
      private static extern int GetWindowLong(IntPtr hWnd, int nIndex);

      [DllImport("user32.dll")]
      private static extern int SetWindowLong(IntPtr hWnd, int nIndex, int dwNewLong);
      private const int GWL_STYLE = -16;
      private const int ES_LEFT = 0x0;
      private const int ES_CENTER = 0x1;
      private const int ES_RIGHT = 0x2;

      /// <summary>
      /// Legt die Textausrichtung einer ComboBox fest.
      /// </summary>
      /// <param name="handle">Window-Handle der ComboBox.</param>
      /// <param name="newStyle">Festzulegender Style.</param>
      internal static void SetTextAlignment(IntPtr handle, int newStyle)
      {
         var info = new COMBOBOXINFO();
         info.cbSize = Marshal.SizeOf(info);
         GetComboBoxInfo(handle, ref info);
         int style = GetWindowLong(info.hwndEdit, GWL_STYLE);
         style = style & ~0x3 | newStyle;
         SetWindowLong(info.hwndEdit, GWL_STYLE, style);
      }
      #endregion
   }

}