﻿using IC_DB;
using System.ComponentModel;
using System.Reflection;

namespace IC_Tester_Controls
{
   /// <summary>
   /// Steuerelement zur Auswahl eines <see cref="PinType"/>.
   /// </summary>
   public partial class PinTypeSelector : UserControl
   {
      /// <summary>Der aktuell ausgewählte <see cref="PinType"/>.</summary>
      public PinType Value = PinType.NC;

      /// <summary>
      /// Initialisiert eine neue Instanz der <see cref="PinTypeSelector"/>-Klasse.
      /// </summary>
      public PinTypeSelector()
      {
         InitializeComponent();
         PinType[] pinModes = Enum.GetValues(typeof(PinType)).Cast<PinType>().ToArray();
         PinModeContextMenu.Items.Clear();

         for (int i = 0; i < pinModes.Length; i++)
         {
            ToolStripItem tsi = new ToolStripMenuItem(GetEnumName(pinModes[i]));
            tsi.Tag = pinModes[i];
            PinModeContextMenu.Items.Add(tsi);
         }
      }

      /// <summary>
      /// Initialisiert eine neue Instanz der <see cref="PinTypeSelector"/>-Klasse.
      /// </summary>
      /// <param name="initialValue">Der Initialwert.</param>
      public PinTypeSelector(PinType initialValue) : this()
      {
         Value = initialValue;
         PinModeBox.Text = GetEnumDescription(Value);
      }

      string GetEnumName(PinType value)
      {
         String? result = Enum.GetName<PinType>(value);
         if (result == null)
            return "??";
         return result;
      }


      /// <summary>
      /// Liest das <see cref="DescriptionAttribute"/> des Enumerationswertes aus.
      /// </summary>
      /// <param name="value">Der Enumerationswerte, dessen <see cref="DescriptionAttribute"/> ermittelt werden soll.</param>
      /// <returns>Der beim Enumerationswert hintelegte Beschreibungstext oder der Name des Enumerationswerts.</returns>
      private static string GetEnumDescription(PinType value)
      {

         FieldInfo? fi = value.GetType().GetField(value.ToString());

         if (fi != null)
         {
            DescriptionAttribute[] attributes = (DescriptionAttribute[])fi.GetCustomAttributes(typeof(DescriptionAttribute), false);
            if (attributes != null && attributes.Length > 0)
            {
               return attributes[0].Description;
            }
         }
         return value.ToString();
      }

      #region [ Events ]
      /// <summary>
      /// Tritt ein, wenn die Selektion geändert wurde.
      /// </summary>
      /// <param name="sender">Die Quelle des Ereignisses.</param>
      /// <param name="pinMode">Der ausgewählte Enumerationswert.</param>
      public delegate void ItemSelectedEventHandler(object sender, PinType pinMode);

      /// <summary>
      /// Tritt ein, wenn die Selektion geändert wurde.
      /// </summary>
      public event ItemSelectedEventHandler? ItemSelected;

      #endregion


      #region "[ Event Handler ]"
      private void PinModeBox_MouseClick(object sender, MouseEventArgs e)
      {
         if (e.Button is MouseButtons.Left)
         {
            PinModeBox.ContextMenuStrip.Show(PinModeBox, new Point(e.X, e.Y));
         }
      }

      Color ColorSave;
      private void PinModeContextMenu_Opening(object sender, CancelEventArgs e)
      {
         ColorSave = BackColor;
         BackColor = Color.LightBlue;
         foreach (ToolStripMenuItem it in PinModeContextMenu.Items)
         {
            it.BackColor = Color.White;
            if ((PinType)it.Tag == Value)
               it.BackColor = Color.LightGray;
         }
      }

      private void PinModeContextMenu_ItemClicked(object sender, ToolStripItemClickedEventArgs e)
      {
         Value = (PinType)e.ClickedItem.Tag;
         PinModeBox.Text = GetEnumDescription(Value);
         ItemSelected?.Invoke(this, Value);
      }
      #endregion

      private void PinModeContextMenu_Closing(object sender, ToolStripDropDownClosingEventArgs e)
      {
         BackColor = ColorSave;
      }
   }
}
